@@ -1,4 +1,4 @@
-/* $Id: terminal-tab-header.c 26424 2007-12-02 14:13:52Z benny $ */
+/* $Id: terminal-tab-header.c 28984 2008-12-06 11:50:22Z stephan $ */
 /*-
  * Copyright (c) 2004-2007 os-cillation e.K.
  *
@@ -27,6 +27,7 @@
 #include <terminal/terminal-preferences.h>
 #include <terminal/terminal-stock.h>
 #include <terminal/terminal-tab-header.h>
+#include <terminal/terminal-screen.h>
 
 
 
@@ -35,6 +36,7 @@ enum
   PROP_0,
   PROP_TAB_POS,
   PROP_TITLE,
+  PROP_ACTIVITY,
 };
 
 enum
@@ -78,19 +80,6 @@ struct _TerminalTabHeaderClass
   void (*set_title)   (TerminalTabHeader *header);
 };
 
-struct _TerminalTabHeader
-{
-  GtkHBox              __parent__;
-
-  TerminalPreferences *preferences;
-
-  GtkWidget           *ebox;
-  GtkWidget           *label;
-
-  /* the popup menu */
-  GtkWidget           *menu;
-};
-
 
 
 static guint header_signals[LAST_SIGNAL];
@@ -134,6 +123,17 @@ terminal_tab_header_class_init (TerminalTabHeaderClass *klass)
                                                         NULL,
                                                         G_PARAM_READWRITE));
 
+  /**
+   * TerminalTabHeader:activity:
+   **/
+  g_object_class_install_property (gobject_class,
+                                   PROP_ACTIVITY,
+                                   g_param_spec_boolean ("activity",
+                                                         "activity",
+                                                         "activity",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE));
+
   /**
    * TerminalTabHeader::close-tab:
    **/
@@ -185,6 +185,8 @@ terminal_tab_header_init (TerminalTabHeader *header)
 
   header->preferences = terminal_preferences_get ();
 
+  header->tab_pos_binding = NULL;
+
   gtk_widget_push_composite_child ();
 
   header->ebox = g_object_new (GTK_TYPE_EVENT_BOX, "border-width", 2, NULL);
@@ -248,6 +250,10 @@ terminal_tab_header_get_property (GObject    *object,
       g_object_get_property (G_OBJECT (header->label), "label", value);
       break;
 
+    case PROP_ACTIVITY:
+      g_object_get_property (G_OBJECT (header), "activity", value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -265,7 +271,9 @@ terminal_tab_header_set_property (GObject      *object,
   TerminalTabHeader *header = TERMINAL_TAB_HEADER (object);
   GtkPositionType    position;
   const gchar       *title;
-
+  gboolean           act = FALSE;
+  GdkColor           act_color;
+  
   switch (prop_id)
     {
     case PROP_TAB_POS:
@@ -292,10 +300,17 @@ terminal_tab_header_set_property (GObject      *object,
 
     case PROP_TITLE:
       title = g_value_get_string (value);
-      terminal_gtk_widget_set_tooltip (header->ebox, title);
+      terminal_gtk_widget_set_tooltip (header->ebox, "%s", title);
       gtk_label_set_text (GTK_LABEL (header->label), title);
       break;
 
+    case PROP_ACTIVITY:
+      act = g_value_get_boolean(value);
+      query_color (header->preferences, "tab-activity-color", &act_color);
+      /* strangely, inactive tab are in state ACTIVE */
+      gtk_widget_modify_fg(header->label, GTK_STATE_ACTIVE, act ? &act_color:NULL);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
