@@ -1,6 +1,6 @@
-/* $Id: terminal-widget.c 134 2005-03-13 16:06:30Z bmeurer $ */
+/* $Id: terminal-widget.c 26427 2007-12-02 14:21:23Z benny $ */
 /*-
- * Copyright (c) 2004-2005 os-cillation e.K.
+ * Copyright (c) 2004-2007 os-cillation e.K.
  *
  * Written by Benedikt Meurer <benny@xfce.org>.
  *
@@ -32,10 +32,8 @@
 
 #include <gdk/gdkkeysyms.h>
 
-#include <libxfce4util/libxfce4util.h>
-
+#include <terminal/terminal-dialogs.h>
 #include <terminal/terminal-enum-types.h>
-#include <terminal/terminal-helper.h>
 #include <terminal/terminal-marshal.h>
 #include <terminal/terminal-preferences.h>
 #include <terminal/terminal-widget.h>
@@ -50,14 +48,17 @@
                         "(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#%]*[^]'.}>) \t\r\n,\\\"])?"
 #define MATCH_BROWSER2  "(www|ftp)[" HOSTCHARS "]*\\.[" HOSTCHARS ".]+(:[0-9]+)?" \
                         "(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#%]*[^]'.}>) \t\r\n,\\\"])?"
+#if !defined(__GLIBC__)
 #define MATCH_MAILER    "(mailto:)?[a-z0-9][a-z0-9.-]*@[a-z0-9][a-z0-9-]*(\\.[a-z0-9][a-z0-9-]*)+"
+#else
+#define MATCH_MAILER    "\\<(mailto:)?[a-z0-9][a-z0-9.-]*@[a-z0-9][a-z0-9-]*(\\.[a-z0-9][a-z0-9-]*)+\\>"
+#endif
 
 
 
 enum
 {
   GET_CONTEXT_MENU,
-  OPEN_URI,
   LAST_SIGNAL,
 };
 
@@ -70,6 +71,8 @@ enum
   TARGET_STRING,
   TARGET_TEXT_PLAIN,
   TARGET_MOZ_URL,
+  TARGET_APPLICATION_X_COLOR,
+  TARGET_GTK_NOTEBOOK_TAB,
 };
 
 
@@ -91,8 +94,7 @@ static gboolean terminal_widget_key_press_event       (GtkWidget            *wid
 static void     terminal_widget_open_uri              (TerminalWidget       *widget,
                                                        const gchar          *link,
                                                        gint                  tag);
-static void     terminal_widget_update_helper_browser (TerminalWidget       *widget);
-static void     terminal_widget_update_helper_mailer  (TerminalWidget       *widget);
+static void     terminal_widget_update_highlight_urls (TerminalWidget       *widget);
 
 
 
@@ -102,9 +104,6 @@ struct _TerminalWidgetClass
 
   /* signals */
   GtkWidget*  (*get_context_menu) (TerminalWidget        *widget);
-  void        (*open_uri)         (TerminalWidget        *widget,
-                                   const gchar           *uri,
-                                   TerminalHelperCategory category);
 };
 
 struct _TerminalWidget
@@ -120,10 +119,11 @@ struct _TerminalWidget
 
 
 
-static GObjectClass *parent_class;
-static guint         widget_signals[LAST_SIGNAL];
+static guint widget_signals[LAST_SIGNAL];
 
-static GtkTargetEntry targets[] =
+
+
+static const GtkTargetEntry targets[] =
 {
   { "text/uri-list", 0, TARGET_URI_LIST },
   { "text/x-moz-url", 0, TARGET_MOZ_URL },
@@ -132,6 +132,8 @@ static GtkTargetEntry targets[] =
   { "COMPOUND_TEXT", 0, TARGET_COMPOUND_TEXT },
   { "STRING", 0, TARGET_STRING },
   { "text/plain", 0, TARGET_TEXT_PLAIN },
+  { "application/x-color", 0, TARGET_APPLICATION_X_COLOR },
+  { "GTK_NOTEBOOK_TAB", GTK_TARGET_SAME_APP, TARGET_GTK_NOTEBOOK_TAB },
 };
 
 
@@ -146,8 +148,6 @@ terminal_widget_class_init (TerminalWidgetClass *klass)
   GtkWidgetClass  *gtkwidget_class;
   GObjectClass    *gobject_class;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = terminal_widget_finalize;
 
@@ -160,26 +160,13 @@ terminal_widget_class_init (TerminalWidgetClass *klass)
    * TerminalWidget::get-context-menu:
    **/
   widget_signals[GET_CONTEXT_MENU] =
-    g_signal_new ("context-menu",
+    g_signal_new (I_("context-menu"),
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (TerminalWidgetClass, get_context_menu),
                   NULL, NULL,
                   _terminal_marshal_OBJECT__VOID,
                   GTK_TYPE_MENU, 0);
-
-  /**
-   * TerminalWidget::open-uri:
-   **/
-  widget_signals[OPEN_URI] =
-    g_signal_new ("open-uri",
-                  G_TYPE_FROM_CLASS (klass),
-                  G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (TerminalWidgetClass, open_uri),
-                  NULL, NULL,
-                  _terminal_marshal_VOID__STRING_ENUM,
-                  G_TYPE_NONE, 2, G_TYPE_STRING,
-                  TERMINAL_TYPE_HELPER_CATEGORY);
 }
 
 
@@ -200,17 +187,14 @@ terminal_widget_init (TerminalWidget *widget)
                      GTK_DEST_DEFAULT_HIGHLIGHT |
                      GTK_DEST_DEFAULT_DROP,
                      targets, G_N_ELEMENTS (targets),
-                     GDK_ACTION_COPY);
+                     GDK_ACTION_COPY | GDK_ACTION_LINK | GDK_ACTION_MOVE);
 
-  /* monitor the helper-* settings */
-  g_signal_connect_swapped (G_OBJECT (widget->preferences), "notify::helper-webbrowser",
-                            G_CALLBACK (terminal_widget_update_helper_browser), widget);
-  g_signal_connect_swapped (G_OBJECT (widget->preferences), "notify::helper-mailreader",
-                            G_CALLBACK (terminal_widget_update_helper_mailer), widget);
+  /* monitor the misc-highlight-urls setting */
+  g_signal_connect_swapped (G_OBJECT (widget->preferences), "notify::misc-highlight-urls",
+                            G_CALLBACK (terminal_widget_update_highlight_urls), widget);
 
-  /* apply the initial helper-* settings */
-  terminal_widget_update_helper_browser (widget);
-  terminal_widget_update_helper_mailer (widget);
+  /* apply the initial misc-highlight-urls setting */
+  terminal_widget_update_highlight_urls (widget);
 }
 
 
@@ -220,14 +204,13 @@ terminal_widget_finalize (GObject *object)
 {
   TerminalWidget *widget = TERMINAL_WIDGET (object);
 
-  /* disconnect the helper-* watchers */
-  g_signal_handlers_disconnect_by_func (G_OBJECT (widget->preferences), G_CALLBACK (terminal_widget_update_helper_browser), widget);
-  g_signal_handlers_disconnect_by_func (G_OBJECT (widget->preferences), G_CALLBACK (terminal_widget_update_helper_mailer), widget);
+  /* disconnect the misc-highlight-urls watch */
+  g_signal_handlers_disconnect_by_func (G_OBJECT (widget->preferences), G_CALLBACK (terminal_widget_update_highlight_urls), widget);
 
   /* disconnect from the preferences */
   g_object_unref (G_OBJECT (widget->preferences));
 
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  (*G_OBJECT_CLASS (terminal_widget_parent_class)->finalize) (object);
 }
 
 
@@ -325,14 +308,14 @@ terminal_widget_context_menu (TerminalWidget *widget,
         }
 
       /* prepend the "COPY" menu item */
-      g_object_set_data_full (G_OBJECT (item_copy), "terminal-widget-link", g_strdup (match), g_free);
+      g_object_set_data_full (G_OBJECT (item_copy), I_("terminal-widget-link"), g_strdup (match), g_free);
       g_signal_connect_swapped (G_OBJECT (item_copy), "activate", G_CALLBACK (terminal_widget_context_menu_copy), widget);
       gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item_copy);
       gtk_widget_show (item_copy);
 
       /* prepend the "OPEN" menu item */
-      g_object_set_data_full (G_OBJECT (item_open), "terminal-widget-link", g_strdup (match), g_free);
-      g_object_set_data_full (G_OBJECT (item_open), "terminal-widget-tag", g_memdup (&tag, sizeof (tag)), g_free);
+      g_object_set_data_full (G_OBJECT (item_open), I_("terminal-widget-link"), g_strdup (match), g_free);
+      g_object_set_data_full (G_OBJECT (item_open), I_("terminal-widget-tag"), g_memdup (&tag, sizeof (tag)), g_free);
       g_signal_connect_swapped (G_OBJECT (item_open), "activate", G_CALLBACK (terminal_widget_context_menu_open), widget);
       gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item_open);
       gtk_widget_show (item_open);
@@ -398,7 +381,7 @@ terminal_widget_button_press_event (GtkWidget       *widget,
 
   if (event->button == 2 && event->type == GDK_BUTTON_PRESS)
     {
-      /* middle-clicking on an URI fires the associated helper */
+      /* middle-clicking on an URI fires the responsible application */
       match = vte_terminal_match_check (VTE_TERMINAL (widget),
                                         event->x / VTE_TERMINAL (widget)->char_width,
                                         event->y / VTE_TERMINAL (widget)->char_height,
@@ -416,7 +399,7 @@ terminal_widget_button_press_event (GtkWidget       *widget,
                                     G_CALLBACK (terminal_widget_commit), &committed);
     }
 
-  GTK_WIDGET_CLASS (parent_class)->button_press_event (widget, event);
+  (*GTK_WIDGET_CLASS (terminal_widget_parent_class)->button_press_event) (widget, event);
 
   if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
     {
@@ -448,11 +431,15 @@ terminal_widget_drag_data_received (GtkWidget        *widget,
                                     guint             time)
 {
   const guint16 *ucs;
+  GdkColor       color;
   GString       *str;
+  GValue         value = { 0, };
   gchar        **uris;
   gchar         *filename;
   gchar         *text;
+  gint           argc;
   gint           n;
+  GtkWidget     *screen;
 
   switch (info)
     {
@@ -460,7 +447,7 @@ terminal_widget_drag_data_received (GtkWidget        *widget,
     case TARGET_UTF8_STRING:
     case TARGET_COMPOUND_TEXT:
     case TARGET_TEXT:
-      text = gtk_selection_data_get_text (selection_data);
+      text = (gchar *) gtk_selection_data_get_text (selection_data);
       if (G_LIKELY (text != NULL))
         {
           if (G_LIKELY (*text != '\0'))
@@ -478,7 +465,7 @@ terminal_widget_drag_data_received (GtkWidget        *widget,
       else
         {
           vte_terminal_feed_child (VTE_TERMINAL (widget),
-                                   selection_data->data,
+                                   (const gchar *) selection_data->data,
                                    selection_data->length);
         }
       break;
@@ -519,29 +506,85 @@ terminal_widget_drag_data_received (GtkWidget        *widget,
         }
       else
         {
-          text = g_strndup (selection_data->data, selection_data->length);
-          uris = g_strsplit (text, "\r\n", 0);
+          /* split the text/uri-list */
+          text = g_strndup ((const gchar *) selection_data->data, selection_data->length);
+          uris = g_uri_list_extract_uris (text);
           g_free (text);
 
-          for (n = 0; uris != NULL && uris[n] != NULL; ++n)
+          /* translate all file:-URIs to quoted file names */
+          for (n = 0; uris[n] != NULL; ++n)
             {
+              /* check if we have a local file here */
               filename = g_filename_from_uri (uris[n], NULL, NULL);
               if (G_LIKELY (filename != NULL))
                 {
+                  /* release the file:-URI */
                   g_free (uris[n]);
-                  uris[n] = filename;
+
+                  /* check if we need to quote the file name (for the shell) */
+                  if (!g_shell_parse_argv (filename, &argc, NULL, NULL) || argc != 1)
+                    {
+                      /* we need to quote the filename */
+                      uris[n] = g_shell_quote (filename);
+                      g_free (filename);
+                    }
+                  else
+                    {
+                      /* no need to quote, shell will handle properly */
+                      uris[n] = filename;
+                    }
                 }
             }
 
-          if (uris != NULL)
-            {
-              text = g_strjoinv (" ", uris);
-              vte_terminal_feed_child (VTE_TERMINAL (widget), text, strlen (text));
-              g_strfreev (uris);
-            }
+          text = g_strjoinv (" ", uris);
+          vte_terminal_feed_child (VTE_TERMINAL (widget), text, strlen (text));
+          g_strfreev (uris);
+          g_free (text);
         }
       break;
+
+    case TARGET_APPLICATION_X_COLOR:
+      if (selection_data->format != 16 || selection_data->length != 8)
+        {
+          g_printerr (_("Received invalid color data: Wrong format (%d) or length (%d)\n"),
+                      selection_data->format, selection_data->length);
+        }
+      else
+        {
+          /* get the color from the selection data (ignoring the alpha setting) */
+          color.red = ((guint16 *) selection_data->data)[0];
+          color.green = ((guint16 *) selection_data->data)[1];
+          color.blue = ((guint16 *) selection_data->data)[2];
+
+          /* prepare the value */
+          g_value_init (&value, GDK_TYPE_COLOR);
+          g_value_set_boxed (&value, &color);
+
+          /* change the background to the specified color */
+          g_object_set_property (G_OBJECT (TERMINAL_WIDGET (widget)->preferences), "color-background", &value);
+
+          /* release the value */
+          g_value_unset (&value);
+        }
+      break;
+
+    case TARGET_GTK_NOTEBOOK_TAB:
+      /* 'send' the drag to the parent widget (TerminalScreen) */
+      screen = gtk_widget_get_parent (widget);
+      if (G_LIKELY (screen))
+        {
+          g_signal_emit_by_name (G_OBJECT (screen), "drag-data-received", context,
+                                 x, y, selection_data, info, time);
+        }
+      break;
+
+    default:
+      /* never finish the drag */
+      return;
     }
+
+  if (info != TARGET_GTK_NOTEBOOK_TAB)
+    gtk_drag_finish (context, TRUE, FALSE, time);
 }
 
 
@@ -551,34 +594,44 @@ terminal_widget_key_press_event (GtkWidget    *widget,
                                  GdkEventKey  *event)
 {
   GtkAdjustment *adjustment = VTE_TERMINAL (widget)->adjustment;
+  gboolean       scrolling_single_line;
+  gboolean       shortcuts_no_menukey;
   gdouble        value;
   gint           x;
   gint           y;
 
+  /* determine current settings */
+  g_object_get (G_OBJECT (TERMINAL_WIDGET (widget)->preferences),
+                "scrolling-single-line", &scrolling_single_line,
+                "shortcuts-no-menukey", &shortcuts_no_menukey,
+                NULL);
+
   /* popup context menu if "Menu" or "<Shift>F10" is pressed */
-  if (event->keyval == GDK_Menu || ((event->state & GDK_SHIFT_MASK) != 0 && event->keyval == GDK_F10))
+  if (event->keyval == GDK_Menu ||
+      (!shortcuts_no_menukey && (event->state & GDK_SHIFT_MASK) != 0 && event->keyval == GDK_F10))
     {
       gtk_widget_get_pointer (widget, &x, &y);
       terminal_widget_context_menu (TERMINAL_WIDGET (widget), 0, event->time, x, y);
       return TRUE;
     }
-  /* scroll up one line with "<Shift>Up" */
-  else if ((event->state & GDK_SHIFT_MASK) != 0
-        && (event->keyval == GDK_Up || event->keyval == GDK_KP_Up))
+  else if (G_LIKELY (scrolling_single_line))
     {
-      gtk_adjustment_set_value (adjustment, adjustment->value - 1);
-      return TRUE;
-    }
-  /* scroll down one line with "<Shift>Down" */
-  else if ((event->state & GDK_SHIFT_MASK) != 0
-        && (event->keyval == GDK_Down || event->keyval == GDK_KP_Down))
-    {
-      value = MIN (adjustment->value + 1, adjustment->upper - adjustment->page_size);
-      gtk_adjustment_set_value (adjustment, value);
-      return TRUE;
+      /* scroll up one line with "<Shift>Up" */
+      if ((event->state & GDK_SHIFT_MASK) != 0 && (event->keyval == GDK_Up || event->keyval == GDK_KP_Up))
+        {
+          gtk_adjustment_set_value (adjustment, adjustment->value - 1);
+          return TRUE;
+        }
+      /* scroll down one line with "<Shift>Down" */
+      else if ((event->state & GDK_SHIFT_MASK) != 0 && (event->keyval == GDK_Down || event->keyval == GDK_KP_Down))
+        {
+          value = MIN (adjustment->value + 1, adjustment->upper - adjustment->page_size);
+          gtk_adjustment_set_value (adjustment, value);
+          return TRUE;
+        }
     }
 
-  return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
+  return (*GTK_WIDGET_CLASS (terminal_widget_parent_class)->key_press_event) (widget, event);
 }
 
 
@@ -588,26 +641,24 @@ terminal_widget_open_uri (TerminalWidget *widget,
                           const gchar    *link,
                           gint            tag)
 {
-  TerminalHelperCategory category;
-  gchar                 *uri;
+  GError *error = NULL;
+  gchar  *uri;
 
+  /* parse the URI from the link */
   if (tag == widget->tag_browser1)
     {
-      category = TERMINAL_HELPER_WEBBROWSER;
       uri = g_strdup (link);
     }
   else if (tag == widget->tag_browser2)
     {
-      category = TERMINAL_HELPER_WEBBROWSER;
       uri = g_strconcat ("http://", link, NULL);
     }
   else if (tag == widget->tag_mailer)
     {
-      category = TERMINAL_HELPER_MAILREADER;
       if (strncmp (link, "mailto:", 7) == 0)
-        uri = g_strdup (link + 7);
-      else
         uri = g_strdup (link);
+      else
+        uri = g_strconcat ("mailto:", link, NULL);
     }
   else
     {
@@ -615,7 +666,13 @@ terminal_widget_open_uri (TerminalWidget *widget,
       return;
     }
 
-  g_signal_emit (G_OBJECT (widget), widget_signals[OPEN_URI], 0, uri, category);
+  /* try to open the URI with the responsible application */
+  if (!exo_url_show_on_screen (uri, NULL, gtk_widget_get_screen (GTK_WIDGET (widget)), &error))
+    {
+      /* tell the user that we were unable to open the responsible application */
+      terminal_dialogs_show_error (widget, error, _("Failed to open the URL `%s'"), uri);
+      g_error_free (error);
+    }
 
   g_free (uri);
 }
@@ -623,12 +680,12 @@ terminal_widget_open_uri (TerminalWidget *widget,
 
 
 static void
-terminal_widget_update_helper_browser (TerminalWidget *widget)
+terminal_widget_update_highlight_urls (TerminalWidget *widget)
 {
-  gchar *setting;
+  gboolean highlight_urls;
 
-  g_object_get (G_OBJECT (widget->preferences), "helper-webbrowser", &setting, NULL);
-  if (exo_str_is_equal (setting, "disabled"))
+  g_object_get (G_OBJECT (widget->preferences), "misc-highlight-urls", &highlight_urls, NULL);
+  if (!highlight_urls)
     {
       if (widget->tag_browser1 >= 0)
         {
@@ -641,6 +698,12 @@ terminal_widget_update_helper_browser (TerminalWidget *widget)
           vte_terminal_match_remove (VTE_TERMINAL (widget), widget->tag_browser2);
           widget->tag_browser2 = -1;
         }
+
+      if (widget->tag_mailer >= 0)
+        {
+          vte_terminal_match_remove (VTE_TERMINAL (widget), widget->tag_mailer);
+          widget->tag_mailer = -1;
+        }
     }
   else
     {
@@ -661,28 +724,7 @@ terminal_widget_update_helper_browser (TerminalWidget *widget)
                                               widget->tag_browser2,
                                               GDK_HAND2);
         }
-    }
-  g_free (setting);
-}
 
-
-
-static void
-terminal_widget_update_helper_mailer (TerminalWidget *widget)
-{
-  gchar *setting;
-
-  g_object_get (G_OBJECT (widget->preferences), "helper-mailreader", &setting, NULL);
-  if (exo_str_is_equal (setting, "disabled"))
-    {
-      if (widget->tag_mailer >= 0)
-        {
-          vte_terminal_match_remove (VTE_TERMINAL (widget), widget->tag_mailer);
-          widget->tag_mailer = -1;
-        }
-    }
-  else
-    {
       if (widget->tag_mailer < 0)
         {
           widget->tag_mailer = vte_terminal_match_add (VTE_TERMINAL (widget),
@@ -692,7 +734,6 @@ terminal_widget_update_helper_mailer (TerminalWidget *widget)
                                               GDK_HAND2);
         }
     }
-  g_free (setting);
 }
 
 
